package taskit.ble232;

import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Editable;
import android.util.Log;
import android.util.SparseArray;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;


public class MyActivity extends Activity {

    private BluetoothGattCharacteristic characteristic_data;
    private static final UUID CONFIG_DESCRIPTOR = UUID.fromString("00002902-0000-1000-8000-00805f9b34fb");
    private static final UUID BLE232_SERVICE_UUID = UUID.fromString("912ffff0-3d4b-11e3-a760-0002a5d5c51b");
    private static final UUID BLE232_CHAR_INTERFACE_UUID = UUID.fromString("912ffff1-3d4b-11e3-a760-0002a5d5c51b");
    private static final UUID BLE232_CHAR_CONFIG_UUID = UUID.fromString("912ffff2-3d4b-11e3-a760-0002a5d5c51b");

    private BluetoothManager bluetoothManager;
    private BluetoothAdapter mBluetoothAdapter;
    private BluetoothGatt gatt;
    private SparseArray<BluetoothDevice> mDevices;
    ArrayAdapter<String> bt_array_adapter;
    private static String address = null;
    int[] rssi_array = new int[100];
    ListView list_devices;
    int mSelectedItem,BLE_STATE;
    Boolean ble_connected = false;
    private String deviceAddress;
    private Context context;
    TextView txt_received;
    EditText edit_text;
    Button button;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        bluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();
        setContentView(R.layout.lay_scan);
        context=this;
        bt_array_adapter = new ArrayAdapter<String>(this, R.layout.lay_custom_list, R.id.scan_name);
        scan_layout_handler.sendEmptyMessage(0);
       discovery_handler.sendEmptyMessage(0);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.my, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private Handler scan_layout_handler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            list_devices = (ListView) findViewById(R.id.list_devices);
            list_devices.setAdapter(bt_array_adapter);
            mDevices = new SparseArray<BluetoothDevice>();
            bt_array_adapter.clear();

            list_devices.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    int itemPosition = position;
                    BluetoothDevice device_info = mDevices.valueAt(itemPosition);
                    address = device_info.getAddress();
                    mSelectedItem = position;
                    bt_array_adapter.notifyDataSetChanged();
                    connect(address);
                }
            });
        }
    };


    private Handler my_activity_layout_handler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            setContentView(R.layout.activity_my);
            txt_received=(TextView)findViewById(R.id.txt_received);
            edit_text=(EditText)findViewById(R.id.edit_text);
            button=(Button)findViewById(R.id.button);
            button.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    String data=edit_text.getText().toString();
                    byte[] byte_data=data.getBytes();
                    write_characteristic(BLE232_SERVICE_UUID, BLE232_CHAR_INTERFACE_UUID, byte_data);
                }
            });

        }

    };

    private android.os.Handler discovery_handler = new android.os.Handler() {
        @Override
        public void handleMessage(Message msg) {
            if (mBluetoothAdapter.isEnabled()) {
                mDevices.clear();
                bt_array_adapter.clear();
                bt_array_adapter.notifyDataSetChanged();
                mBluetoothAdapter.startLeScan(mLeScanCallback);
            } else {
                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivity(enableBtIntent);
            }
        }
    };

    private android.os.Handler stop_discovery = new android.os.Handler() {
        @Override
        public void handleMessage(Message msg) {
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
        }
    };

    private final BluetoothAdapter.LeScanCallback mLeScanCallback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public void onLeScan(final BluetoothDevice device, int rssi, byte[] scanRecord) {
                mDevices.put(device.hashCode(), device);
                int dev_pos = mDevices.indexOfValue(device);
                rssi_array[dev_pos] = rssi;
                list_handler.sendMessage(Message.obtain(null, dev_pos, rssi));
        }
    };

    private android.os.Handler list_handler = new android.os.Handler() {
        @Override
        public void handleMessage(Message msg) {
            list_devices = (ListView) findViewById(R.id.list_devices);
            bt_array_adapter.clear();
            for (int i = 0; i < mDevices.size(); i++) {
                BluetoothDevice device_info = mDevices.valueAt(i);
                int rssi_value = rssi_array[i];
                bt_array_adapter.add(device_info.getName() + "\r\n" + device_info.getAddress() + "\r\n" + "RSSI=" + String.valueOf(rssi_value) + "dBm" +"\r\n");
                bt_array_adapter.notifyDataSetChanged();
            }
            bt_array_adapter.notifyDataSetChanged();
        }
    };




    private void connect(String address) {
        if (!ble_connected) {
            if (mBluetoothAdapter == null || address == null) {
                toast_handler.sendMessage(Message.obtain(null, 0, " BluetoothAdapter not initialized or unspecified address. "));
            }
            if (deviceAddress != null && address.equals(deviceAddress) && gatt != null) {
                toast_handler.sendMessage(Message.obtain(null, 0, "Trying to use an existing BluetoothGatt for connection."));
            }
            final BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
            if (device == null) {
                toast_handler.sendMessage(Message.obtain(null, 0, "Device not found.  Unable to connect."));
            }
            if (device != null) {
                mBluetoothAdapter.stopLeScan(mLeScanCallback);
                deviceAddress = address;
                gatt = device.connectGatt(this, false, btGattCallback);
            }
        }else  if (ble_connected){
            disconnect.sendEmptyMessage(0);
        }
    }

    private android.os.Handler toast_handler = new android.os.Handler() {
        @Override
        public void handleMessage(Message msg) {
            Toast toast_1 = Toast.makeText(getApplicationContext(), msg.obj.toString(), Toast.LENGTH_SHORT);
            toast_1.setGravity(Gravity.CENTER | Gravity.CENTER, 0, 0);
            toast_1.show();
        }
    };

    private android.os.Handler disconnect = new android.os.Handler() {
        @Override
        public void handleMessage(Message msg) {
            try {
                //setRefreshAction_connecting(false);
                //action_connecting.sendMessage(Message.obtain(null, 0, "false"));
                if (BLE_STATE == BluetoothProfile.STATE_CONNECTED) {
                    gatt.disconnect();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    private BluetoothGattCallback btGattCallback = new BluetoothGattCallback() {
        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            super.onConnectionStateChange(gatt, status, newState);
            BLE_STATE = newState;
            if (newState == BluetoothProfile.STATE_CONNECTED) {
                ble_connected = true;
                toast_handler.sendMessage(Message.obtain(null, 0, "CONNECTED"));
                BLE_STATE = newState;
                list_handler.sendMessage(Message.obtain(null, 0, 0));
                stop_discovery.sendEmptyMessage(0);
                gatt.discoverServices();
                my_activity_layout_handler.sendEmptyMessage(0);
            }
            if (newState == BluetoothProfile.STATE_DISCONNECTED) {
                ble_connected = false;
                list_handler.sendMessage(Message.obtain(null, 0, 0));
                toast_handler.sendMessage(Message.obtain(null, 0, "DISCONNECTED"));
                gatt.close();
            }
            if (newState == BluetoothProfile.STATE_CONNECTING) {

            }
            if (newState == BluetoothProfile.STATE_DISCONNECTING) {

            }
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                set_notify.sendMessage(Message.obtain(null,0,0));
            } else {
                toast_handler.sendMessage(Message.obtain(null, 0, "Service discovery failed"));
            }
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);
            final byte[] data;
            data = characteristic.getValue();
            gatt.readRemoteRssi();
            if (data != null && data.length > 0) {
                final StringBuilder stringBuilder = new StringBuilder(data.length);
                for (byte byteChar : data)
                    stringBuilder.append(String.format("%02X ", byteChar));
                String aString = new String(data);
                data_handler.sendMessage(Message.obtain(null, 0, aString));
            }
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            //toast_handler.sendMessage(Message.obtain(null, 0, "the value was written"));
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            super.onCharacteristicRead(gatt, characteristic, status);
            if (status == BluetoothGatt.GATT_SUCCESS) {
                final byte[] data;
                data = characteristic.getValue();
                if (data != null && data.length > 0) {
                    final StringBuilder stringBuilder = new StringBuilder(data.length);
                    for (byte byteChar : data)
                        stringBuilder.append(String.format("%02X ", byteChar));
                    /*
                    if (Sensor_Type.equals(characteristic.getUuid())) {
                        sensor_type_value=data;
                    }
                    if (pH_Calibration.equals(characteristic.getUuid())) {
                        pH_Calibration_value=data;
                    }
                    */
                }
            }
        }

        @Override
        public void onDescriptorWrite(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                //Log.e(TAG, "GATT_SUCCESS");
            } else
            if (status == BluetoothGatt.GATT_INSUFFICIENT_AUTHENTICATION) {
                toast_handler.sendMessage(Message.obtain(null, 0, "GATT_INSUFFICIENT_AUTHENTICATION"));
                } else if (status==BluetoothGatt.GATT_WRITE_NOT_PERMITTED) {
                toast_handler.sendMessage(Message.obtain(null, 0, "GATT_WRITE_NOT_PERMITTED"));
            }
        };

    };

    private android.os.Handler data_handler = new android.os.Handler() {
        @Override
        public void handleMessage(Message msg) {
            txt_received.setText(msg.obj.toString());
        }
    };

    public void write_characteristic(UUID service_uuid,UUID characteristic_uuid, byte[] value) {
        BluetoothGattService service;
        service = gatt.getService(service_uuid);
        if (service == null) {

        }
        BluetoothGattCharacteristic characteristic;
        characteristic = service.getCharacteristic(characteristic_uuid);
        if (characteristic == null) {

        }
        characteristic.setValue(value);
        gatt.writeCharacteristic(characteristic);
    }

    private Handler set_notify=new Handler(){
        @Override
        public void handleMessage(Message msg) {
            try{
                if (gatt!=null){
                    BluetoothGattCharacteristic characteristic;
                    characteristic = gatt.getService(BLE232_SERVICE_UUID).getCharacteristic(BLE232_CHAR_INTERFACE_UUID);
                    //Enable local notifications
                    gatt.setCharacteristicNotification(characteristic, true);
                    //Enabled remote notifications
                    BluetoothGattDescriptor desc = characteristic.getDescriptor(CONFIG_DESCRIPTOR);
                    desc.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
                    gatt.writeDescriptor(desc);
                }
            }
            catch (Exception e) {
                Log.d("", "Error: " + e.toString());
            }
        }
    };

    @Override
    protected void onDestroy() {
        stop_discovery.sendEmptyMessage(0);
        gatt.close();
        System.exit(0);
    }

    @Override
    public void onBackPressed() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setIcon(android.R.drawable.ic_dialog_alert);
        builder.setTitle("Quit?");
        builder.setMessage("Do you want to close the Application");
        builder.setPositiveButton("yes", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int id) {
                finish();
            }
        });
        builder.setNegativeButton("no",new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog,int id) {
                dialog.cancel();
            }
        });
        AlertDialog alertDialog = builder.create();
        alertDialog.show();
    }
}
